package ubz.model;

import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import ubz.persistence.DotazioneLoader;

public class Cassiere {

	private Disponibilit disponibilit;

	public Cassiere(DotazioneLoader loader) {
		if (loader == null)
			throw new IllegalArgumentException("loader nullo");
		this.disponibilit = loader.getDisponibilit();
	}

	public int getDisponibilitTaglio(Taglio t){
		return disponibilit.getQuantit(t);
	} 
	
	public Prelievo preleva(int importo) throws ImpossibleWithdrawException{
		SortedMap<Taglio, Integer> mappaPrelievo = new TreeMap<>();
		int importoResiduo = importo;
		for(Taglio t: Taglio.values()){
			 int quantitTaglioCorrente = calcolaQuantitDiQuestoTaglio(t, importoResiduo);
			 importoResiduo -= quantitTaglioCorrente * t.getValore();
			 mappaPrelievo.put(t, quantitTaglioCorrente);
		}
		if (importoResiduo>0) 
			throw new ImpossibleWithdrawException(importo, importoResiduo);
		else {
			aggiornaDisponibilitInCassa(mappaPrelievo);
			return new Prelievo(mappaPrelievo);
		}
	}

	protected void aggiornaDisponibilitInCassa(Map<Taglio, Integer> mappaPrelievo) {
		for(Taglio t: Taglio.values()){
			int disponibilitAttuale = getDisponibilitTaglio(t);
			int banconoteDaPrelevare = mappaPrelievo.get(t);
			disponibilit.aggiorna(t, disponibilitAttuale-banconoteDaPrelevare);
		}	
	}

	/* versione base SENZA POLITICHE */
	protected int calcolaQuantitDiQuestoTaglio(Taglio t, int importo){
		 int quantitRichiesta = importo/t.getValore();
		 int quantitDisponibile = disponibilit.getQuantit(t);
		 return Math.min(quantitRichiesta, quantitDisponibile);
	}
}
